% function [alpha1, alpha2, Cutoff,alfa] = CGB_convert(lambda,rho_p,Pbar)
% Convert the channel gain based (CGB) power control law parameters

% lambda: channel gain parameter, lambda = 1/E(g). Assume the marginal distribution
% of the amplitude of the channel gain is Rayleigh. The pdf of channel gain g is 
%             f_{pdf}(g) = lambda * exp( - lambda  * g)

% rho_p, Pbar: parameters for the combined time-domain water filling (TDWF)
% and truncated channel inversion (TCI) power control law.
% Define the TDWF power control law as P_{TDWF}(g)
%             P_{TDWF}(g) = 1/C - 1/g     if g > C
%                         = 0             if g < C
% And the TCI power control law as P_{TCI}(g)
%             P_{TCI}(g) = alfa/g     if g > C
%                         = 0          if g < C
% Notice that the two power control laws have the same cutoff threshold C,
% which can be obtained by the average power constraint
%             E_g{ P_{TDWF}(g) } = Pbar.
% Knowing C, the parameter for the TCI PC alfa can be also obtained by the
% average power constraint
%             E_g{ P_{TCI}(g) } = Pbar.
% Finally, the combined TDWF and TCI power control law is defined as:
%    f(g) = rho_p * P_{TDWF}(g) + ( 1- rho_p ) * P_{TCI}    
          
% alpha1, alpha2, Cutoff are the parameters for the more general power 
% control law:
%            f(g) = alpha1 + alpha2 / g    (g > C)
%                 = 0                      (g < C) 
% The average power constraint
%            E_g{ f(g) } = Pbar
% raises the following constraint on the parameters:
%            alpha1*exp(-lambda*Cutoff)+lambda*alpha2*expint(Cutoff*lambda)-Pbar

% If cutoff threshold C can not be found, all of the output parameters will 
% be set to -1. In this case, the third input parameters of the function 
% water_filling need to be reset to a larger range.

function [alpha1, alpha2, Cutoff,alfa] = CGB_convert(lambda,rho_p,Pbar)

Cutoff = water_filling(Pbar, lambda,  [1e-10,40]); %cutoff threshold
if Cutoff == -1
    alpha1 = -1;
    alpha2 = -1;
    alfa = -1;
else
    alfa = inversion_get_alfa(Pbar, lambda, Cutoff); %gain of TCI
    alpha1 = rho_p / Cutoff;
    alpha2 = alfa * ( 1 - rho_p ) - rho_p;
end